#!/usr/bin/python3
import rospy
from sensor_msgs.msg import NavSatFix
import json
from tools import meters2lat_lon, lat_lon2meters
import numpy as np

def normalize(v):
    norm = np.linalg.norm(v)
    if norm == 0:
       return v
    return v / norm

if __name__ == "__main__":
    rospy.init_node("gps_streamer")
    pub = rospy.Publisher("dji_osdk_ros/gps_position", NavSatFix, queue_size=10)
    drone_id = rospy.get_param("drone_id")
    updateRate = rospy.Rate(20)

    while not rospy.has_param('/robot_list/robot_' + str(drone_id) + '/waypoints'):
        rospy.loginfo("waiting for parameter...")
        updateRate.sleep()
    waypointDict = rospy.get_param("/robot_list/robot_" + str(drone_id) + "/waypoints")
    index = 0
    seq = 0
    vecPosition = np.array([waypointDict["waypoint_" + str(index)][1], waypointDict["waypoint_" + str(index)][2]])
    # anchorPoint = [37.223162, -80.432770]
    # Pi = 3.1415924

    rate = rospy.Rate(10) # publish at 10hz
    while not rospy.is_shutdown():
        target = waypointDict["waypoint_" + str(index)]

        vecTarget = np.array([target[1], target[2]])
        vecPosition = vecPosition + 0.000001*normalize(vecTarget - vecPosition)

        # check if close to target
        if np.linalg.norm(vecTarget - vecPosition) < 0.000005 and index < (len(waypointDict) - 1):
            # made it to waypoint, pick next one`
            index = index + 1

        gpsMsg = NavSatFix()
        # gpsMsg.header = 0
        gpsMsg.header.seq = seq
        # gpsMsg.header.stamp.secs = int(rospy.get_time())
        # gpsMsg.header.frame_id = 'gps'

        gpsMsg.latitude = vecPosition[0]
        gpsMsg.longitude = vecPosition[1]
        gpsMsg.altitude = 5.0

        pub.publish(gpsMsg)

        seq += 1
        updateRate.sleep()
