#!/usr/bin/python3

import rospy
import numpy as np
from sensor_msgs.msg import NavSatFix
from sensor_msgs.msg import Imu
from std_msgs.msg import Float32
from geometry_msgs.msg import PointStamped
from geometry_msgs.msg import QuaternionStamped
from geometry_msgs.msg import Transform
from geometry_msgs.msg import Vector3
from geometry_msgs.msg import Quaternion
# from scipy.spatial.transform import Rotation
from tools import lat_lon2meters, meters2lat_lon
import math
from pyproj import Transformer

def point_rotation(origin, pt, ang):
    # returns the pt rotated about the origin by ang (ang in degrees)
    c = math.cos(math.radians(ang))
    s = math.sin(math.radians(ang))
    # translate to origin
    pt_temp = [pt[0] - origin[0], pt[1] - origin[1]]
    pt_spun = [ pt_temp[0]*c - pt_temp[1]*s, pt_temp[0]*s + pt_temp[1]*c ]
    # translate back to frame
    pt_spun = [pt_spun[0] + origin[0], pt_spun[1] + origin[1]]
    return pt_spun


def meters2lat_lon(meters_point):
    # returns point converted to lat lon from meters coordinate system
    transformer = Transformer.from_crs(3857, 4326)
    latlon_point = transformer.transform(meters_point[0], meters_point[1])
    return latlon_point


def lat_lon2meters(latlon_point):
    # returns point converted to meters from lat lon coordinate system
    transformer = Transformer.from_crs(4326, 3857)
    meters_point = transformer.transform(latlon_point[0], latlon_point[1])
    return meters_point


def heightCallback(data):
    global height
    height = data.data


def rotCallback(data):
    global quat
    quat = data.quaternion


def gpsCallback(data):
    global quat
    global height
    # convert and compare against home point
    anchorPoint = rospy.get_param("~anchorPoint")
    apMeters = lat_lon2meters([anchorPoint[0], anchorPoint[1]])
    cpMeters = lat_lon2meters([data.latitude, data.longitude])
    currentOffset = [cpMeters[0] - apMeters[0], cpMeters[1] - apMeters[1]]

    # performing dark magics
    translation = Vector3()
    translation.x = currentOffset[1]
    translation.y = currentOffset[0]
    translation.z = height

    transform = Transform()
    transform.translation = translation
    transform.rotation = quat
    posePublisher.publish(transform)

# standing in for the feedback qualisys stream
posePublisher = rospy.Publisher("feedback", Transform, queue_size = 1)
quat = Quaternion()
height = 0.0

def main():
    rospy.init_node("qualisysSpoof")
    anchorPoint = rospy.get_param("~anchorPoint")
    # dumb just use name spaces
    rospy.Subscriber("dji_osdk_ros/gps_position", NavSatFix, gpsCallback, queue_size = 1)
    rospy.Subscriber("dji_osdk_ros/attitude", QuaternionStamped, rotCallback, queue_size = 1)
    rospy.Subscriber("dji_osdk_ros/height_above_takeoff", Float32, heightCallback, queue_size = 1)
    rospy.spin()
    # get anchor point from params

if __name__ == '__main__':
    main()
