from pyproj import Transformer
import math

def meters2lat_lon(meters_point):
    # returns point converted to lat lon from meters coordinate system
    transformer = Transformer.from_crs(3857, 4326)
    latlon_point = transformer.transform(meters_point[0], meters_point[1])
    return latlon_point

def lat_lon2meters(latlon_point):
    # returns point converted to meters from lat lon coordinate system
    transformer = Transformer.from_crs(4326, 3857)
    meters_point = transformer.transform(latlon_point[0], latlon_point[1])
    return meters_point

def normalizeAngle(angle):
    newAngle = angle%360
    newAngle = (newAngle + 360)%360
    if(newAngle >= 180):
        newAngle -= 360
    return newAngle

def point_rotation(origin, pt, ang):
    # returns the pt rotated about the origin by ang (ang in degrees)
    c = math.cos(math.radians(ang))
    s = math.sin(math.radians(ang))
    # translate to origin
    pt_temp = [pt[0] - origin[0], pt[1] - origin[1]]
    pt_spun = [ pt_temp[0]*c - pt_temp[1]*s, pt_temp[0]*s + pt_temp[1]*c ]
    # translate back to frame
    pt_spun = [pt_spun[0] + origin[0], pt_spun[1] + origin[1]]
    return pt_spun
